
/* ==========================================================================
**
**                            ListEditor.c
**
** 1991 WILLISoft
**
** ==========================================================================
*/

#include "ListEditor.h"
#include "ListEditorClass.h"
#include "EmbossedGadgetClass.h"
#include <stdlib.h>
#include <string.h>
#include "minmax.h"
#ifndef __GNUC__
#include <clib/exec_protos.h>
#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>
#endif
#ifdef __GNUC__
#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/graphics.h>
#endif
#ifdef __SASC
#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/graphics.h>
#endif
#include "amigamem.h"

#define STATE_ADDING  2
#define STATE_EDITING 1
#define STATE_NOTHING 0

#define BROWSE &self->browse
#define EDIT   &self->edit
#define ADD    &self->add
#define DELETE &self->delete


   void ListEditor_CleanUp( ListEditor *self )
   {
      CleanUp( (PObject *)BROWSE );
      CleanUp( (PObject *)EDIT );
      CleanUp( (PObject *)ADD );
      CleanUp( (PObject *)DELETE );
   }

   USHORT ListEditor_nGadgets( ListEditor *self )
   {
      return (USHORT)( nGadgets( (Interactor *)BROWSE ) + nGadgets( (Interactor *)EDIT )
         + nGadgets( (Interactor *)ADD ) + nGadgets( (Interactor *)DELETE ) );
   }

   Gadget *ListEditor_FirstGadget( ListEditor *self )
   {
      return FirstGadget( (Interactor *)BROWSE );
   }

   #define ADD_X 60
   #define ADD_Y 14
   #define Y_DIVIDER 2

   Point ListEditor_AskSize( ListEditor *self,
                             PIXELS      Width,
                             PIXELS      Height )
   {
      Point size, min_size, browse_size, edit_size;
      PIXELS browse_y;

      browse_size = MinSize( (GraphicObject *)BROWSE );
      edit_size   = MinSize( (GraphicObject *)EDIT );

      min_size.x = MAX( browse_size.x, edit_size.x );
      min_size.x = MAX( min_size.x, ADD_X*2 );
      min_size.y = browse_size.y + Y_DIVIDER + edit_size.y + Y_DIVIDER + ADD_Y;

      size.x = MAX( min_size.x, Width );
      size.y = MAX( min_size.y, Height );

      /* At this point we know that 'size' is at least the minimum size,
      ** but there may be more size constraints than that.
      ** (ScrollingLists currently must be an even font size.)
      */

      browse_y    = size.y - ADD_Y - Y_DIVIDER - edit_size.y - Y_DIVIDER;
      browse_size = AskSize( (GraphicObject *)BROWSE, size.x, browse_y );

      size.x  = browse_size.x;
      size.y  = browse_size.y + Y_DIVIDER + edit_size.y + Y_DIVIDER + ADD_Y;

      return size;
   }


   Point ListEditor_SetSize( ListEditor *self,
                             PIXELS      Width,
                             PIXELS      Height )
   {
      Point size, edit_size;
      PIXELS browse_y;

      size = AskSize( (GraphicObject *)self, Width, Height );

      edit_size = SetSize( (GraphicObject *)EDIT, size.x, 0 );
      browse_y  = size.y - ADD_Y - Y_DIVIDER - edit_size.y - Y_DIVIDER;

      SetSize( (GraphicObject *)BROWSE, size.x, browse_y );

      self->slist.Size = size;
      SetLocation( (GraphicObject *)self, self->slist.Location.x, self->slist.Location.y );

      return size;
   }

   Point ListEditor_SetLocation( ListEditor *self,
                                 PIXELS      LeftEdge,
                                 PIXELS      TopEdge )
   {
      Point size, location, browse_size, edit_size;

      size        = Size( (GraphicObject *)self );
      browse_size = Size( (GraphicObject *)BROWSE );
      edit_size   = Size( (GraphicObject *)EDIT );

      SetLocation( (GraphicObject *)BROWSE, LeftEdge, TopEdge );
      SetLocation( (GraphicObject *)EDIT, LeftEdge, TopEdge + browse_size.y + Y_DIVIDER );

      SetLocation( (GraphicObject *)ADD,    LeftEdge,                TopEdge + size.y-ADD_Y);
      SetLocation( (GraphicObject *)DELETE, LeftEdge + size.x-ADD_X, TopEdge + size.y-ADD_Y );

      location.x           = LeftEdge;
      location.y           = TopEdge;
      self->slist.Location = location;

      return location;
   }



   void ListEditor_Render( ListEditor  *self,
                           RastPort       *RPort )
   {
      Render( (GraphicObject *)BROWSE, RPort );
      Render( (GraphicObject *)EDIT,   RPort );
      Render( (GraphicObject *)ADD,    RPort );
      Render( (GraphicObject *)DELETE, RPort );
   }


   const StringList *ListEditor_StringList_of( ListEditor *self )
   {
      return StringList_of( (StringLister *)BROWSE );
   }

   /* Changed to BOOL from void -- EDB */
   BOOL ListEditor_AddString( ListEditor  *self,
                              char        *entry,
                              UBYTE        qualifier )
   {
      /* Added return -- EDB */
      return( AddString( (StringLister *)BROWSE, entry, qualifier ) );
   }


   /* Changed to BOOL from void -- EDB */
   BOOL ListEditor_DeleteString( ListEditor  *self,
                                 USHORT       entry )
   {
      const StringList *slist;

      slist = StringList_of( (StringLister *)self );
      /* Added return -- EDB */
      return( DeleteString( (StringLister *)BROWSE, entry ) );
   }

   /* Changed to BOOL from void -- EDB */
   BOOL ListEditor_DeleteAllStrings( ListEditor  *self )
   {


      /* Added return -- EDB */
      return( DeleteAllStrings( (StringLister *)BROWSE ) );
   }


   void ListEditor_SelectString( ListEditor  *self,
                                 USHORT       entry,
                                 BOOL         Select   )
   {
      SelectString( (StringLister *)BROWSE, entry, Select );
   }

                          /* Changed from ListEditor -- EDB */
   void ListEditor_Refresh( Interactor *us )
   {
      ListEditor *self = NULL;
      self = (ListEditor *)us;
      Refresh( (Interactor *)BROWSE );
      Refresh( (Interactor *)EDIT );
      Refresh( (Interactor *)ADD );
      Refresh( (Interactor *)DELETE );
   }

   USHORT ListEditor_ClaimEvent(  ListEditor     *self,
                                  IntuiMessage   *event )
   {
      return( (USHORT) ( ClaimEvent( (Interactor *)BROWSE, event )
           | ClaimEvent( (Interactor *)EDIT,   event )
           | ClaimEvent( (Interactor *)ADD,    event )
           | ClaimEvent( (Interactor *)DELETE, event ) ) );
   }


   #define CLAIM_EVENT(b,e) ( &b->g == (Gadget *)e->IAddress )

   USHORT ListEditor_Respond( ListEditor   *self,
                              IntuiMessage *event )
   {
      const StringList  *slist;
      USHORT       i;
      char        *string, *val;

      USHORT browse_response  =0;
      USHORT edit_response    =0;
      USHORT add_response     =0;
      USHORT delete_response  =0;

      slist = StringList_of( (StringLister *)self );
      if( ( browse_response = Respond( (Interactor *)BROWSE,event ) ) & CHANGED_STATE )
      {
         for( i = 0; i < slist->nEntries; i++ )
         {
            if( slist->Qualifiers[i] & ENTRY_SELECTED )
            {
               EnableIactor( (Interactor *)EDIT, TRUE );
               EnableIactor( (Interactor *)DELETE, TRUE );
               SetStringValue( (Valuator *)EDIT, slist->Entries[i] );
               self->s     = i;
               self->state = STATE_EDITING;
               Refresh( (Interactor *)EDIT );
               Refresh( (Interactor *)DELETE );
               Activate( (Interactor *)EDIT,TRUE );
               break;
            }
         }
      }
      else if( ( edit_response = Respond( (Interactor *)EDIT,event ) ) & DEACTIVATED )
      {
         if( val = StringValue( (Valuator *)EDIT ) )
         {
            if( self->state == STATE_ADDING )
            {
               if( strlen( val ) >0 )
               {
                  AddString( (StringLister *)self, val, 0 );
               }
            }
            else if( self->state == STATE_EDITING )
            {
               if( strlen( val ) > 0 )
               {
                  if( string = Astrdup( val ) )
                  {
                     Afree( slist->Entries[self->s] );
                     slist->Entries[self->s] = string;
                  }
               }
               else
               {
                  DeleteString( (StringLister *)self, self->s );
               }
            }
         }
         SetStringValue( (Valuator *)EDIT, NULL );
         EnableIactor( (Interactor *)EDIT, FALSE );
         EnableIactor( (Interactor *)DELETE, FALSE );
         self->state = STATE_NOTHING;
         edit_response |= CHANGED_STATE;
         SelectString( (StringLister *)self, self->s, FALSE );
         Refresh( (Interactor *)EDIT );
         Refresh( (Interactor *)DELETE );
         Refresh( (Interactor *)BROWSE );
      }
      else if( (add_response = Respond( (Interactor *)ADD,event ) ) & CHANGED_STATE )
      {
         if( self->state == STATE_EDITING )
         {
            SelectString( (StringLister *)self, self->s, FALSE );
            Refresh( (Interactor *)BROWSE );
         }
         self->state = STATE_ADDING;
         EnableIactor( (Interactor *)EDIT, TRUE );
         SetStringValue( (Valuator *)EDIT, NULL );
         Refresh( (Interactor *)EDIT );
         Activate( (Interactor *)EDIT,TRUE );
      }

      else if( ( delete_response = Respond( (Interactor *)DELETE,event ) ) & CHANGED_STATE )
      {
         self->state = STATE_NOTHING;
         DeleteString( (StringLister *)self, self->s );
         EnableIactor( (Interactor *)DELETE, FALSE );
         SetStringValue( (Valuator *)EDIT, NULL );
         EnableIactor( (Interactor *)EDIT, FALSE );
         Refresh( (Interactor *)self );
      }

      return( (USHORT)
         ( browse_response | edit_response | add_response | delete_response ) );

   }

   BOOL ListEditor_Activate( ListEditor *self, BOOL activate )
   {
      return Activate( (Interactor *)EDIT, activate );
   }

   void ListEditor_SetInteractorWindow( ListEditor *self,
                                        pcgWindow  *window )
   {
      self->slist.IaWindow = window;
      SetInteractorWindow( (Interactor *)BROWSE, window );
      SetInteractorWindow( (Interactor *)ADD,    window );
      SetInteractorWindow( (Interactor *)EDIT,   window );
      SetInteractorWindow( (Interactor *)DELETE, window );
   }

   ULONG ListEditor_IDCMPFlags( ListEditor *self )
   {
      return IDCMPFlags( (Interactor *)BROWSE ) | IDCMPFlags( (Interactor *)EDIT )
       | IDCMPFlags( (Interactor *)ADD ) | IDCMPFlags( (Interactor *)DELETE );
   }

   BOOL ListEditor_EnableIactor( ListEditor *self, BOOL enable )
   {
      if( enable )
      {
         EnableIactor( (Interactor *)BROWSE, TRUE );
         EnableIactor( (Interactor *)EDIT,   ( self->state ));
         EnableIactor( (Interactor *)ADD,    TRUE );
         EnableIactor( (Interactor *)DELETE, ( self->state == STATE_EDITING ) );
      }
      else
      {
         EnableIactor( (Interactor *)BROWSE, FALSE );
         EnableIactor( (Interactor *)EDIT,   FALSE );
         EnableIactor( (Interactor *)ADD,    FALSE );
         EnableIactor( (Interactor *)DELETE, FALSE );
      }
      return enable;
   }

   char *ListEditor_Title( ListEditor *self )
   {
      return Title( (GraphicObject *)EDIT );
   }

   BOOL ListEditor_SetTitle( ListEditor *self, char *title )
   {
      return SetTitle( (GraphicObject *)EDIT, title );
   }


   void ListEditor_Init( ListEditor *self,
                         PIXELS      LeftEdge,
                         PIXELS      TopEdge,
                         PIXELS      Width,
                         PIXELS      Height,
                         pcg_3DPens  pens,
                         char       *label )
   {

      StringLister_Init( &self->slist );

      self->slist.isa = ListEditorClass();
      self->state     = STATE_NOTHING;
      self->s         = 0;


      ScrollingList_Init( BROWSE, 0,0,0,0, pens, FALSE );
      StringGadget_Init( EDIT, 0,0,0, 80, pens, label );
      BoolGadget_Init( ADD, 0, 0, ADD_X, ADD_Y, pens, "Add" );
      BoolGadget_Init( DELETE, 0, 0, ADD_X, ADD_Y, pens, "Delete" );

      ChainGadgets( FirstGadget( (Interactor *)BROWSE ), FirstGadget( (Interactor *)EDIT )   );
      ChainGadgets( FirstGadget( (Interactor *)EDIT ),   FirstGadget( (Interactor *)ADD )    );
      ChainGadgets( FirstGadget( (Interactor *)ADD ),    FirstGadget( (Interactor *)DELETE ) );
      SetLocation( (GraphicObject *)self, LeftEdge, TopEdge );
      SetSize( (GraphicObject *)self, Width, Height );

      EnableIactor( (Interactor *)self, TRUE );
   }



   BOOL ListEditor_elaborated = FALSE;

   struct StringListerClass ListEditor_Class;

   void ListEditorClass_Init( struct StringListerClass *class )
   {
      StringListerClass_Init( class );
      class->isa         = StringListerClass();
      class->CleanUp     = (void(*)(PObject *))ListEditor_CleanUp;
      class->ClassName   = "ListEditor";
      class->AskSize     = (Point(*)(GraphicObject *, PIXELS, PIXELS))ListEditor_AskSize;
      class->SetSize     = (Point(*)(GraphicObject *, PIXELS, PIXELS))ListEditor_SetSize;
      class->SizeFlags   = GraphicObject_SizeFlagsAll;
      class->SetLocation = (Point(*)(GraphicObject *, PIXELS, PIXELS))ListEditor_SetLocation;
      class->Title       = (char*(*)(GraphicObject *))ListEditor_Title;
      class->SetTitle    = (BOOL(*)(GraphicObject *, char *))ListEditor_SetTitle;
      class->Respond     = (USHORT(*)(Interactor *, IntuiMessage *))ListEditor_Respond;
      class->Refresh     = (void(*)(Interactor *))ListEditor_Refresh;
      class->Render      = (void(*)(GraphicObject *, RastPort *))ListEditor_Render;
      class->ClaimEvent  = (USHORT(*)(Interactor *, IntuiMessage *))ListEditor_ClaimEvent;
      class->nGadgets    = (USHORT(*)(Interactor *))ListEditor_nGadgets;
      class->FirstGadget = (Gadget*(*)(Interactor *))ListEditor_FirstGadget;
      class->SetInteractorWindow = (void(*)(Interactor *, struct pcgWindow *))ListEditor_SetInteractorWindow;
      class->IDCMPFlags  = (ULONG(*)(Interactor *))ListEditor_IDCMPFlags;
      class->isEnabled   = (BOOL(*)(Interactor *))EmbossedGadget_isEnabled;
      class->EnableIactor = (BOOL(*)(Interactor *, BOOL))ListEditor_EnableIactor;
      class->Activate    = (BOOL(*)(Interactor *, BOOL))ListEditor_Activate;
      class->StringList_of = (StringList*(*)(StringLister *))ListEditor_StringList_of;
      class->AddString     = (BOOL(*)(StringLister *, char *, UBYTE))ListEditor_AddString;
      class->DeleteString  = (BOOL(*)(StringLister *, USHORT))ListEditor_DeleteString;
      class->DeleteAllStrings = (BOOL(*)(StringLister *))ListEditor_DeleteAllStrings;
      class->SelectString  = (void(*)(StringLister *, USHORT, BOOL))ListEditor_SelectString;

   }



   struct StringListerClass *ListEditorClass( void )
   {
      if( ! ListEditor_elaborated )
      {
         ListEditorClass_Init( &ListEditor_Class );
         ListEditor_elaborated = TRUE;
      }

      return &ListEditor_Class;
   }


